@@ -44,7 +44,10 @@ import shutil,zipfile
 import networkx as nx
 import six,shutil
 from urllib.error import HTTPError
-
+from yapmap import *
+import tarfile,gzip
+import multiprocessing as mp
+import time,logging,math
 
 DEFAULT_CORPUS = 'TxtLab'
 DEFAULT_CORPUS_ID = 'txtlab'
@@ -59,7 +62,7 @@ HOME=os.path.expanduser("~")
 MODERNIZE_SPELLING=False
 config = tools.config
 
-ZIP_PART_DEFAULTS={'txt','freqs','metadata','xml','raw','data'}
+ZIP_PART_DEFAULTS={'txt','freqs','metadata','xml','data'} # raw
 DOWNLOAD_PART_DEFAULTS=['metadata','freqs','txt','data']
 PREPROC_CMDS=['txt','freqs','mfw','dtm']
 DEST_LLTK_CORPORA=config.get('CLOUD_DEST','/Share/llp_corpora')
@@ -67,8 +70,19 @@ DEFAULT_MFW_N =25000
 DEFAULT_DTM_N = 25000
 DEFAULT_MFW_YEARBIN = 100
 MANIFEST_REQUIRED_DATA=['name','id']
+TEXT_PATH_KEYS=[
+	'path_freqs',
+	'path_txt',
+	'path_xml'
+]
+
+import logging
+logging.basicConfig(format='%(message)s', level=logging.INFO)
+
 
 MANIFEST_DEFAULTS=dict(
+	# id='corpus',
+	# name='Corpus',
 	path_txt='txt',
 	path_xml='xml',
 	path_index='',
@@ -153,6 +167,10 @@ PATH_MANIFESTS = tools.remove_duplicates([
 ], remove_empty=True)
 #print(PATH_MANIFESTS)
 
+EMPTY_GROUP='(all)'
+
+
+
 from pprint import pprint
 
 
