@@ -1,5 +1,6 @@
 from lltk.imports import *
 import six,shutil
+from yapmap import *
 
 HOME=expanduser("~")
 LLTK_ROOT = LIT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
@@ -50,7 +51,7 @@ try:
 		with open(file, mode) as f:
 			f.write(cell)
 		get_ipython().run_cell(cell)
-except NameError:
+except (NameError,ModuleNotFoundError):
 	pass
 
 
@@ -399,9 +400,14 @@ def get_ocr_corrections(lang='en'):
 		OCRCORREX[lang]=d
 	return OCRCORREX[lang]
 
+def get_encoding(fn):
+	import chardet
+	with open(fn, 'rb') as f:
+		result = chardet.detect(f.read())  # or readline if the file is large
+	return result['encoding']
 
 
-def save_df(df,ofn,move_prev=False,index=None,key='',log=print):
+def save_df(df,ofn,move_prev=False,index=None,key='',log=print,verbose=True):
 	import pandas as pd
 	if os.path.exists(ofn) and move_prev: iter_move(ofn)
 	ext = os.path.splitext(ofn.replace('.gz',''))[-1][1:]
@@ -427,7 +433,7 @@ def save_df(df,ofn,move_prev=False,index=None,key='',log=print):
 		# try again as csv?
 		ofn=os.path.splitext(ofn)[0]+'.csv'
 		df.to_csv(ofn)
-	if log is not None: log('>> saved:',ofn)
+	if verbose: print('Saved:',ofn)
 
 
 def read_df(ifn,key='',**attrs):
@@ -547,9 +553,10 @@ def cleanstrip(x):
 
 	return x
 
-def detokenize(x,clean=True):
+def detokenize(x):
 	# x=basic_detokenizer(x)
 	x=detokenize_treebank(x)
+	x=x.replace(' .','.')
 	# if clean: x=cleanstrip(x)
 	return x
 
@@ -694,8 +701,7 @@ def readgen_csv(fnfn,sep=None,encoding='utf-8',errors='ignore',header=[],progres
 	if not sep: sep=',' if fnfn.endswith('csv') or fnfn.endswith('.csv.gz') else '\t'
 	if progress and not num_lines:
 		with open(fnfn,encoding=encoding,errors=errors) as f:
-			for _ in tqdm(f,desc='Counting lines'):
-				num_lines+=1
+			for _ in f: num_lines+=1
 	
 	with open(fnfn,encoding=encoding,errors=errors) as f:
 		# csv_reader = reader(f)
@@ -712,31 +718,18 @@ def readgen_csv(fnfn,sep=None,encoding='utf-8',errors='ignore',header=[],progres
 				except Exception:
 					pass
 
-def readgen(fnfn,header=None,tsep='\t',keymap={},keymap_all=six.text_type,encoding='utf-8',as_list=False,as_tuples=False,as_dict=True,toprint=True,progress=True):
-	if 'jsonl' in fnfn.split('.'):
-		for dx in readgen_jsonl(fnfn):
-			yield dx
+def readgen(fnfn,**y):
+	ext=os.path.splitext(fnfn)[-1]
+	if ext=='.jsonl':
+		yield from readgen_jsonl(fnfn,**y)
+	elif ext=='.csv':
+		yield from readgen_csv(fnfn,**y)
+	elif ext=='.txt':
+		yield from readgen_csv(fnfn,sep='\t',**y)
 	else:
-		import time
-		now=time.time()
-
-		"""
-		if tsep=='\t' and toprint:
-			print('>> streaming as tsv:',fnfn)
-		elif tsep==',' and toprint:
-			print('>> streaming as csv:',fnfn)
-		"""
-
-		if progress:
-			num_lines = get_num_lines(fnfn)
-			from tqdm import tqdm
-			_fn=os.path.basename(fnfn)
-			for dx in tqdm(readgen_csv(fnfn),total = num_lines, desc=f'>> reading csv ({_fn})'): yield dx
-		else:
-			for dx in readgen_csv(fnfn): yield dx
-
-		nownow=time.time()
-		if not progress and toprint: print('   done ['+str(round(nownow-now,1))+' seconds]')
+		# print(f'[readgen()] Resorting to non-generator load for {fnfn}')
+		df=read_df(fnfn)
+		yield from df.to_dict('records')
 
 def header(fnfn,tsep='\t',encoding='utf-8'):
 	header=[]
@@ -1422,7 +1415,8 @@ def gleanPunc2(aToken):
 
 	return (aPunct0, aToken, aPunct1)
 
-def modernize_spelling_in_txt(txt,spelling_d):
+def modernize_spelling_in_txt(txt,spelling_d=None):
+	if not spelling_d: spelling_d=get_spelling_modernizer()
 	lines=[]
 	for ln in txt.split('\n'):
 		ln2=[]
@@ -1703,6 +1697,8 @@ def download_wget(url, save_to, **attrs):
 def download(url,save_to,force=False,desc=''):
 	here=os.getcwd()
 	if not force and os.path.exists(save_to): return
+	savedir=os.path.dirname(save_to)
+	if not os.path.exists(savedir): os.makedirs(savedir)
 	try:
 		download_pycurl(url,save_to,desc=desc)
 	except (ImportError,ModuleNotFoundError) as e:
@@ -1834,7 +1830,7 @@ class Bunch(object):
 			yield v
 
 
-
+def mask_home_dir(path): return ppath(path)
 def ppath(path):
 	import os
 	return path.replace(
@@ -1896,7 +1892,7 @@ def unzip(zipfn, dest='.', flatten=False, overwrite=False, replace_in_filenames=
 	from tqdm import tqdm
 
 	# Open your .zip file
-	if not desc: desc=f'Extracting {os.path.basename(zipfn)}'
+	if not desc: desc=f'Extracting {os.path.basename(zipfn)} to {dest}'
 	with ZipFile(zipfn) as zip_file:
 		namelist=zip_file.namelist()
 
@@ -1917,11 +1913,14 @@ def unzip(zipfn, dest='.', flatten=False, overwrite=False, replace_in_filenames=
 				pass
 			except FileNotFoundError:
 				continue
-			
-			with zip_file.open(member) as source, open(target_fnfn,'wb') as target:
-				shutil.copyfileobj(source, target)
-
-
+			try:
+				with zip_file.open(member) as source, open(target_fnfn,'wb') as target:
+					shutil.copyfileobj(source, target)
+			except FileNotFoundError:
+				print('!! File not found:',target_fnfn)
+def safesample(df,n,replace=False):
+    if replace: return df.sample(n=n,replace=True)
+    return df.sample(n=n) if len(df)>n else df
 
 
 def get_num_lines(filename):
@@ -2028,6 +2027,20 @@ def check_move_file(src,dst):
 	except (KeyboardInterrupt,EOFError) as e:
 		return False
 
+def check_move_link_file(src,dst):
+	src=os.path.abspath(src)
+	dst=os.path.abspath(dst)
+	try:
+		if check_make_dir(os.path.dirname(dst)):
+			if input(f'\nMove and link\n    {src}\nto\n    {dst}\n[Y/n] ').strip()!='n':
+				print('\n>> moving {src} to {dst}')
+				shutil.copyfile(src,dst)
+				os.unlink(src)
+				print(f'>> linking {src} to {dst}')
+				os.symlink(dst,src)
+	except (KeyboardInterrupt,EOFError) as e:
+		return False
+
 
 def check_make_dirs(paths,ask=True):
 	l=[]
@@ -2132,133 +2145,6 @@ def remove_duplicates(seq,remove_empty=False):
 
 
 
-
-
-
-def pmap_df(df, func, num_proc=DEFAULT_NUM_PROC):
-	df_split = np.array_split(df, num_proc)
-	df = pd.concat(pmap(func, df_split, num_proc=num_proc))
-	return df
-
-def pmap_do(inp):
-	func,obj,args,kwargs = inp
-	return func(obj,*args,**kwargs)
-
-def pmap_iter(func, objs, args=[], kwargs={}, num_proc=DEFAULT_NUM_PROC, use_threads=False, progress=True, desc=None, **y):
-	"""
-	Yields results of func(obj) for each obj in objs
-	Uses multiprocessing.Pool(num_proc) for parallelism.
-	If use_threads, use ThreadPool instead of Pool.
-	Results in any order.
-	"""
-	
-	# imports
-	import multiprocessing as mp
-	from tqdm import tqdm
-	
-	# check num proc
-	num_cpu = mp.cpu_count()
-	if num_proc>num_cpu: num_proc=num_cpu
-
-	# if parallel
-	if not desc: desc=f'Mapping {func.__name__}()'
-	if desc: desc=f'{desc} [x{num_proc}]'
-	if num_proc>1 and len(objs)>1:
-
-		# real objects
-		objects = [(func,obj,args,kwargs) for obj in objs]
-
-		# create pool
-		pool=mp.Pool(num_proc) if not use_threads else mp.pool.ThreadPool(num_proc)
-
-		# yield iter
-		iterr = pool.imap(pmap_do, objects)
-		
-		for res in tqdm(iterr,total=len(objects),desc=desc) if progress else iterr:
-			yield res
-
-		# Close the pool?
-		pool.close()
-		pool.join()
-	else:
-		# yield
-		for obj in (tqdm(objs,desc=desc) if progress else objs):
-			yield func(obj,*args,**kwargs)
-
-def pmap(*x,**y):
-	"""
-	Non iterator version of pmap_iter
-	"""
-	# return as list
-	return list(pmap_iter(*x,**y))
-
-
-
-def do_pmap_group(obj,*x,**y):
-	import pandas as pd
-	# unpack
-	func,group_df,group_key,group_name = obj
-	# load from cache?
-	if type(group_df)==str:
-		group_df=pd.read_pickle(group_df)
-	# run func
-	outdf=func(group_df,*x,**y)
-	# annotate with groupnames on way out
-	if type(group_name) not in {list,tuple}:group_name=[group_name]
-	for x,y in zip(group_key,group_name):
-		outdf[x]=y
-	# return
-	return outdf
-
-def pmap_groups(func,df_grouped,use_cache=True,num_proc=DEFAULT_NUM_PROC,**attrs):
-	import os,tempfile,pandas as pd
-	from tqdm import tqdm
-
-	# get index/groupby col name(s)
-	group_key=df_grouped.grouper.names
-	# if not using cache
-	# if not use_cache or attrs.get('num_proc',1)<2:
-	if not use_cache or len(df_grouped)<2 or num_proc<2:
-		objs=[
-			(func,group_df,group_key,group_name)
-			for group_name,group_df in df_grouped
-		]
-	else:
-		objs=[]
-		tmpdir=tempfile.mkdtemp()
-		for i,(group_name,group_df) in enumerate(tqdm(list(df_grouped),desc='Preparing input')):
-			tmp_path = os.path.join(tmpdir, str(i)+'.pkl')
-			# print([i,group_name,tmp_path,group_df])
-			group_df.to_pickle(tmp_path)
-			objs+=[(func,tmp_path,group_key,group_name)]
-
-	# desc?
-	if not attrs.get('desc'): attrs['desc']=f'Mapping {func.__name__}'
-
-
-	return pd.concat(
-		pmap(
-			do_pmap_group,
-			objs,
-			num_proc=num_proc,
-			**attrs
-		)
-	).set_index(group_key)
-
-
-
-def plot_distro(x,data,y='..count..'):
-	import plotnine as p9
-	fig=p9.ggplot(p9.aes(x=x,y=y), data=data)
-	fig+=p9.geom_density()
-	return fig
-
-
-
-
-
-
-
 ### UTILS
 
 
